﻿using Microsoft.AspNetCore.Http;
using Ocelot.LoadBalancer.LoadBalancers;
using Ocelot.Responses;
using Ocelot.Values;
using System.Collections.Generic;
using System.Threading.Tasks;
using System;
using Ocelot.DownstreamRouteFinder.Finder;
using System.Linq;

namespace ApiGatewayDemo
{
    /// <summary>
    /// 自定义负载均衡策略
    /// </summary>
    public class CustomerBalancer : ILoadBalancer
    {
        private readonly Func<Task<List<Service>>> _services;
        private readonly object _lock = new object();

        private int _last;
        public CustomerBalancer(Func<Task<List<Service>>> services)
        {
            _services = services;
        }

        public async Task<Response<ServiceHostAndPort>> Lease(HttpContext httpContext)
        {
            var services = await _services();

            if (services == null || services.Count == 0)
            {
                return null;
            }

            lock (_lock)
            {
                var healthyServices = services;

                if (services.Count == 0)
                {
                    return null;
                }

                var totalWeight = services.Sum(s => int.Parse(s.Version));
                var randomWeight = new Random().Next(totalWeight);

                var selectedService = services.First();
                foreach (var serviceItem in services)
                {
                    if (randomWeight < int.Parse(serviceItem.Version))
                    {
                        selectedService = serviceItem;
                        break;
                    }

                    randomWeight -= int.Parse(serviceItem.Version);
                }

                return new OkResponse<ServiceHostAndPort>(new ServiceHostAndPort(selectedService.HostAndPort.DownstreamHost, selectedService.HostAndPort.DownstreamPort));
            }
        }

        public void Release(ServiceHostAndPort hostAndPort)
        {

        }
    }
}
